#ifdef _PROTOTYPE_

#include "common.hh"

class Vector
{
public:
    float x, y;

    Vector () {}
    Vector (float x_, float y_) : x(x_), y(y_)
    {
    }

    float dot (Vector v) {return x * v.x + y * v.y;}
    float cross (Vector v) {return x * v.y - y * v.x;}
    float len () {return sqrtf(x * x + y * y);}
    Vector add (Vector v) {return Vector(x + v.x, y + v.y);}
    Vector sub (Vector v) {return Vector(x - v.x, y - v.y);}
    Vector scale (float s) {return Vector(x * s, y * s);}
    Vector operator + (Vector v) {return add(v);}
    Vector operator - (Vector v) {return sub(v);}
    float operator ^ (Vector v) {return cross(v);}
    float operator | (Vector v) {return dot(v);}
    Vector operator * (float s) {return scale(s);}
};

class Rod
{
public:
    float x, y;
    float dx, dy;
    RGB rgb;
    Rod *next, *prev;

    void init()
    {
        next = NULL;
        prev = NULL;
    }

    Rod (float x_, float y_, float dx_, float dy_, RGB rgb_) :
        x(x_), y(y_), dx(dx_), dy(dy_), rgb(rgb_)
    {
        init();
    }

    bool is_left(Vector &v)
    {
        Vector a(x - v.x, y - v.y);
        return (a ^ v) > 0.0;
    }
    bool is_right(Vector &v)
    {
        Vector a(x - v.x, y - v.y);
        return (a ^ v) < 0.0;
    }

    bool Rod::intersection (Rod const *rod, Vector *inter);
    bool Rod::intersection2(Rod const *rod, Vector *inter);
};

class Soup
{
public:
    Rod *first, *last;
    Soup()
    {
        first = NULL;
        last = NULL;
    }

    ~Soup()
    {
        Rod *next;
        for (Rod *r = first; r; r = next)
        {
            next = r->next;
            delete r;
        }
    }

    void add(Rod *rod)
    {
        rod->next = NULL;
        rod->prev = last;
        if (last)
            last->next = rod;
        else
            first = rod;

        last = rod;
    }

    void add_random (int n);
};

#endif /* _PROTOTYPE_ */

#include "rodsoup.hh"

void Soup::add_random (int n)
{
    for (int i = 0; i < n; i++)
    {
        int x = 10;
        int y = 10;
        int w = SCREEN_W - 20;
        int h = SCREEN_H - 20;
        int x1 = x + rand() % w;
        int y1 = y + rand() % h;
        int x2 = x + rand() % w;
        int y2 = y + rand() % h;
        RGB rgb = {rand() % 256, rand() % 256, rand() % 256, rand() % 256};
        Rod *rod = new Rod (x1, y1, x2 - x1, y2 - y1, rgb);
        add(rod);
    }
}

/* The intersection of two lines means:
   a0 + i a = b0 + j b with i,j in [0..1]
   c = b0 - a0
   i a = c + j b

   Solve for i and j:

   i ax = cx + j bx (1)
   i ay = cy + j by (2)

   i = (cx + j bx) / ax (1')
   ay (cx + j bx) / ax = cy + j by (1' in 2)
   ay cx + j ay bx = ax cy + j ax by
   j = (ax cy - ay cx) / (ay bx - ax by)
   j = (a x c) / (b x a)

   j = (i ay - cy) / by (2')
   i ax = cx + bx (i ay - cy) / by (2' in 1)
   i ax by - cx by = i bx ay - by cy
   i = (cx by - by cy) / (ax by - bx ay)
   i = (c x b) / (a x b)
   
   i = (c x b) / (a x b)
   j = (c x a) / (a x b)
   
   i < 0 or i > 1 means, no collision
   j < 0 or j > 1 means, no collision
   
   The actual point is:
   a0 + i a = b0 + j b
   a0 + a * (c x b) / (a x b) = b0 + b * (c x a) / (a x b)

   This is how we do the actual checking in the code:
   ab = a x b
   ca = c x a
   cb = c x b

   Remember:
   i = cb / ab
   j = ca / ab
   
   ab = 0 means, the lines are parallel
   
   i < 0 means: (ab > 0 && cb < 0) || (ab < 0 && cb > 0)
   i > 1 means: (ab > 0 && cb > ab) || (ab < 0 && cb < ab)
   j < 0 means: (ab > 0 && ca < 0) || (ab < 0 && ca > 0)
   j > 1 means: (ab > 0 && ca > ab) || (ab < 0 && ca < ab)
*/

bool
Rod::intersection(Rod const *rod, Vector *inter)
{
    Vector a(dx, dy);
    Vector b(rod->dx, rod->dy);
    Vector c(rod->x - x, rod->y - y);

    float ab = a ^ b;
    float ca = c ^ a;
    float cb = c ^ b;

    if (ab == 0) /* parallel */
        return false;

    if (ab < 0)
    {
        if (ca > 0 || cb > 0)
            return false;
        if (ca < ab || cb < ab)
            return false;
    }
    else
    {
        if (ca < 0 || cb < 0)
            return false;
        if (ca > ab || cb > ab)
            return false;
    }
    if (inter)
    {
        *inter = Vector (x, y) + a * (cb / ab);
    }
    return true;
}

/* This makes the passed rod extending indefinitely into its direction.
 */
bool
Rod::intersection2(Rod const *rod, Vector *inter)
{
    Vector a(dx, dy);
    Vector b(rod->dx, rod->dy);
    Vector c(rod->x - x, rod->y - y);

    float ab = a ^ b;
    float ca = c ^ a;
    float cb = c ^ b;

    if (ab == 0) /* parallel */
        return false;

    if (ab < 0)
    {
        if (ca > 0 || cb > 0)
            return false;
        if (cb < ab)
            return false;
    }
    else
    {
        if (ca < 0 || cb < 0)
            return false;
        if (cb > ab)
            return false;
    }
    if (inter)
    {
        *inter = Vector (x, y) + a * (cb / ab);
    }
    return true;
}

